@@ -19,10 +19,11 @@ package provider
 import (
 	"context"
 	"fmt"
+	"math"
 	"time"
 
-	"github.com/kubernetes-incubator/custom-metrics-apiserver/pkg/provider"
-	"github.com/kubernetes-incubator/custom-metrics-apiserver/pkg/provider/helpers"
+	"github.com/kubernetes-sigs/custom-metrics-apiserver/pkg/provider"
+	"github.com/kubernetes-sigs/custom-metrics-apiserver/pkg/provider/helpers"
 	pmodel "github.com/prometheus/common/model"
 	apierr "k8s.io/apimachinery/pkg/api/errors"
 	apimeta "k8s.io/apimachinery/pkg/api/meta"
@@ -33,7 +34,7 @@ import (
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/dynamic"
-	"k8s.io/klog"
+	"k8s.io/klog/v2"
 	"k8s.io/metrics/pkg/apis/custom_metrics"
 
 	prom "github.com/directxman12/k8s-prometheus-adapter/pkg/client"
@@ -83,6 +84,12 @@ func (p *prometheusProvider) metricFor(value pmodel.SampleValue, name types.Name
 		return nil, err
 	}
 
+	var q *resource.Quantity
+	if math.IsNaN(float64(value)) {
+		q = resource.NewQuantity(0, resource.DecimalSI)
+	} else {
+		q = resource.NewMilliQuantity(int64(value*1000.0), resource.DecimalSI)
+	}
 	return &custom_metrics.MetricValue{
 		DescribedObject: ref,
 		Metric: custom_metrics.MetricIdentifier{
@@ -90,7 +97,7 @@ func (p *prometheusProvider) metricFor(value pmodel.SampleValue, name types.Name
 		},
 		// TODO(directxman12): use the right timestamp
 		Timestamp: metav1.Time{time.Now()},
-		Value:     *resource.NewMilliQuantity(int64(value*1000.0), resource.DecimalSI),
+		Value:     *q,
 	}, nil
 }
 
@@ -118,8 +125,8 @@ func (p *prometheusProvider) metricsFor(valueSet pmodel.Vector, info provider.Cu
 	}, nil
 }
 
-func (p *prometheusProvider) buildQuery(info provider.CustomMetricInfo, namespace string, names ...string) (pmodel.Vector, error) {
-	query, found := p.QueryForMetric(info, namespace, names...)
+func (p *prometheusProvider) buildQuery(info provider.CustomMetricInfo, namespace string, metricSelector labels.Selector, names ...string) (pmodel.Vector, error) {
+	query, found := p.QueryForMetric(info, namespace, metricSelector, names...)
 	if !found {
 		return nil, provider.NewMetricNotFoundError(info.GroupResource, info.Metric)
 	}
@@ -140,9 +147,9 @@ func (p *prometheusProvider) buildQuery(info provider.CustomMetricInfo, namespac
 	return *queryResults.Vector, nil
 }
 
-func (p *prometheusProvider) GetMetricByName(name types.NamespacedName, info provider.CustomMetricInfo) (*custom_metrics.MetricValue, error) {
+func (p *prometheusProvider) GetMetricByName(name types.NamespacedName, info provider.CustomMetricInfo, metricSelector labels.Selector) (*custom_metrics.MetricValue, error) {
 	// construct a query
-	queryResults, err := p.buildQuery(info, name.Namespace, name.Name)
+	queryResults, err := p.buildQuery(info, name.Namespace, metricSelector, name.Name)
 	if err != nil {
 		return nil, err
 	}
@@ -171,7 +178,7 @@ func (p *prometheusProvider) GetMetricByName(name types.NamespacedName, info pro
 	return p.metricFor(resultValue, name, info)
 }
 
-func (p *prometheusProvider) GetMetricBySelector(namespace string, selector labels.Selector, info provider.CustomMetricInfo) (*custom_metrics.MetricValueList, error) {
+func (p *prometheusProvider) GetMetricBySelector(namespace string, selector labels.Selector, info provider.CustomMetricInfo, metricSelector labels.Selector) (*custom_metrics.MetricValueList, error) {
 	// fetch a list of relevant resource names
 	resourceNames, err := helpers.ListObjectNames(p.mapper, p.kubeClient, namespace, selector, info)
 	if err != nil {
@@ -181,7 +188,7 @@ func (p *prometheusProvider) GetMetricBySelector(namespace string, selector labe
 	}
 
 	// construct the actual query
-	queryResults, err := p.buildQuery(info, namespace, resourceNames...)
+	queryResults, err := p.buildQuery(info, namespace, metricSelector, resourceNames...)
 	if err != nil {
 		return nil, err
 	}
